
class SearchFilters:
    @staticmethod
    def Users():
        return "users"

    @staticmethod
    def Latest():
        return "latest"

    @staticmethod
    def Photos():
        return "photos"

    @staticmethod
    def Videos():
        return "videos"

